$wnd.rstudio.runAsyncCallback17("function WOt(a){this.a=a}\nfunction p$t(a){this.a=a}\nfunction r$t(a,b){this.a=a;this.b=b}\nfunction WEv(a,b){this.a=a;this.b=b}\nfunction qFv(a,b){this.a=a;this.b=b}\nfunction RUw(a,b){QUw();this.b=a;this.a=b}\nfunction FVt(a,b){oUt();var c;c=a;frv(c,b.a)}\nfunction BDw(a,b){Vqk(a.u,b)}\nfunction $qv(a,b){BDw(a.ub,b)}\nfunction iNt(a){ryi(a.n,a.c,null)}\nfunction jNt(a){ryi(a.n,a.g,null)}\nfunction QNt(a){fOt(a.o,a.g,eHA,null)}\nfunction VNt(a){fOt(a.o,a.g,'next',null)}\nfunction XNt(a){fOt(a.o,a.g,'prev',null)}\nfunction YNt(a){fOt(a.o,a.g,'refresh',null)}\nfunction kNt(a){ryi(a.n,a.g+'?print-pdf',null)}\nfunction Osj(a){return {items:a}}\nfunction Tqv(a,b){gn((_m(),$m),new qFv(a,b))}\nfunction EVt(a,b){oUt();gn((_m(),$m),new r$t(b,a))}\nfunction mTh(a){$wnd.desktop.setPresentationUrl(a)}\nfunction Vqk(a,b){a.d.navigateToPresentationEditorLocation(b)}\nfunction frv(a,b){JYw(a.w,Mmz,false)?ppv(a,new WEv(a,b)):RJv(a._,b)}\nfunction cNt(a,b){TBe(b.substr(0,4),epy)||(b=qvm(a.p,b));return b}\nfunction hNt(a){!!a.b&&U1h(a.k,new RUw((zmf(),Rmf(a.b.source_file,false,-1,0)),Isj(a.d,a.a,a.e)),false)}\nfunction Jsj(a,b){var c,d,e,f;b==-1&&(b=a.auto_slide_level);f=Hsj(a,b,true);for(c=0;c<f.length;c++){e=f[c];for(d=0;d<e.items.length;d++){if(TBe(e.items[d].type,B_y))return c}}return 0}\nfunction Isj(a,b,c){var d,e,f,g,h;c==-1&&(c=b.auto_slide_level);h=Hsj(b,c,false);if(a<h.length){g=h[a];zl(g.items,Lsj(g.items[0].row))}e=[];for(d=0;d<h.length;d++){g=h[d];for(f=0;f<g.items.length;f++){xl(e,g.items[f])}}return Ksj(e,b.auto_slide_level)}\nfunction RJv(a,b){var c,d,e,f,g,h,i;e=QJv(a).items;d=[];for(g=0;g<e.length;g++){h=e[g];TBe(h.type,B_y)||(d[d.length]=h,undefined)}c=-1;i=b.items;for(f=0;f<i.length;f++){if(TBe(i[f].type,B_y)){c=f;break}}if(c>=0){h=d[c];!!h&&XOu(a.a,P$w(h.row,0),true)}}\nfunction Hsj(a,b,c){var d,e,f,g,h,i;i=[];g=a.items;for(d=0;d<g.length;d++){f=g[d];e=i.length==0||TBe(f.type,wmy)||TBe(f.type,'hr')||TBe(f.type,Vly)&&f.level<=b;if(e){h=Osj([]);xl(h.items,f);i[i.length]=h}else (c||!TBe(f.type,B_y))&&xl(i[i.length-1].items,f)}return i}\nfunction lNt(a,b){var c,d;c=b.a;d=cNt(a,c.url);a.o=Jsj(b.a.editor_state,b.a.slide_level);a.o==0&&a.d!=0&&(a.o=a.d);Ipq(a.j);if(!TBe(d,a.g)||!a.j.o){a.g=d;a.c=d;a.b=c.quarto_navigation;a.a=c.editor_state;a.e=c.slide_level;(hSh()||!!$wnd.remoteDesktop)&&mTh(a.g);UNt(a.j,a.g,a.b)}else{YNt(a.j)}}\nfunction UNt(a,b,c){var d;(hSh()||!!$wnd.remoteDesktop)&&(P5e(b)?(Oh(a.d).removeAttribute(a_y),undefined):(Oh(a.d).setAttribute(a_y,WGB),undefined));a.b=b;a.o=null;a.g=null;d=Qh(a.d);d>0&&yi(a,new okf(gO(d*0.6666666666666666)+((a.K?Ph(a.K):0)+(a.O?Ph(a.O):0))));Nee(a.d,a.b);c.is_website?zom(a.i):yom(a.i,c.source_file,c.output_file);zlh(a.p)}\nfunction SOt(a){try{if(a.c<0)return;if(!a.a.a){a.a.a=vPt(a.a.b.a);Liq(a.a,a.a.a)}switch(a.c){case 1:a.a.a.J.b7b();break;case 2:a.a.a.J.c7b();break;case 3:hNt(a.a.a);break;case 4:QNt(a.a.a.j);break;case 5:VNt(a.a.a.j);break;case 6:iNt(a.a.a);break;case 7:jNt(a.a.a);break;case 8:XNt(a.a.a.j);break;case 9:kNt(a.a.a);break;case 10:lNt(a.a.a,a.b[0]);break;case 11:YNt(a.a.a.j);break;case 12:a.a.a.J.d7b();break;case 13:a.a.a.J.e7b();}}finally{a.c<=0&&a.b[0]!=null&&a.b[0].ln()}}\nqNd(304,1,G8y);_.jn=function VOt(){SOt((new WOt(this)).a)};qNd(7480,1,EAy,WOt);_.ln=function XOt(){SOt(this.a)};var gRc=hAe(7480);qNd(3290,1,zAy,p$t);_.pw=function q$t(a){EVt(this.a,a)};var $Uc=hAe(3290);qNd(3289,1,_my,r$t);_.ln=function s$t(){FVt(this.a,this.b)};var _Uc=hAe(3289);qNd(5061,1,EAy,WEv);_.ln=function XEv(){Tqv(this.a,this.b)};var o9c=hAe(5061);qNd(5060,1,_my,qFv);_.ln=function rFv(){$qv(this.a,this.b)};var z9c=hAe(5060);qNd(3335,Koy,{},RUw);_.Zn=function SUw(a){var b,c;b=(c=Y8i(a.k,this.b),!!c&&_N(c,78)?c:(S8i(),M8i));o2t(a.b,this.b,b,new p$t(this))};_._n=function UUw(){return PUw};_.$n=function TUw(){return PUw};var Cqd=hAe(3335);xky(hm)(17);\n//# sourceURL=rstudio-17.js\n")
