$wnd.rstudio.runAsyncCallback16("function nMt(a){this.a=a}\nfunction FJt(a,b){this.a=a;this.b=b;mmf.call(this)}\nfunction hJt(a,b){a.a=b;BLt(a.p,dJt(a,null),a.a.file_path)}\nfunction jJt(a,b){b.slide_index==0&&Ipq(a.p);a.a=b;BLt(a.p,dJt(a,null),a.a.file_path)}\nfunction qJt(a){loh(bJt.Xpc(),poe());U1h(a.c,new P3h,false)}\nfunction eJt(a){var b;b=yzi(new zzi(0,bJt.Tpc()));uum(a.n,new FJt(a,b))}\nfunction uum(a,b){xzm(a,Epz,'close_presentation_pane',new wL,b)}\nfunction pJt(a){var b;a.J.d7b();!!a.a&&a.a.active&&(b=mjf(Oh(a.p.d).contentWindow),!(!TBe(oEy,b)&&!TBe('javascript:void(0)',b)))&&hJt(a,a.a)}\nfunction jMt(a){var b,c;try{if(a.c<0)return;if(!a.a.a){a.a.a=ZMt(a.a.b.a);Liq(a.a,a.a.a)}switch(a.c){case 1:eJt((c=a.a.a,a.b[0],c));break;case 2:jJt(a.a.a,a.b[0]);break;case 3:a.a.a.J.b7b();break;case 4:a.a.a.J.c7b();break;case 5:pJt(a.a.a);break;case 6:qJt((b=a.a.a,a.b[0],b));break;case 7:a.a.a.J.e7b();}}finally{a.c<=0&&a.b[0]!=null&&a.b[0].ln()}}\nvar eKB='Closing Presentation...',fKB='Opening Presentation...';qNd(8207,33,{},FJt);_.AA=function HJt(a){var b;U1h((b=this,a,b).a.c,new P3h,false)};_.zA=function GJt(a){Oyi(this.b,a.a?a.a.c:a.d)};var rRc=hAe(8207);qNd(9072,1,UfB);_.Tpc=function kKt(){return eKB};_.Xpc=function pKt(){return fKB};qNd(9070,1,UfB);_.Tpc=function CKt(){return eKB};_.Xpc=function HKt(){return fKB};qNd(9071,1,UfB);_.Tpc=function UKt(){return 'Fermeture de la pr\\xE9sentation...'};_.Xpc=function ZKt(){return 'Ouvrir la pr\\xE9sentation...'};qNd(465,1,G8y);_.jn=function mMt(){jMt((new nMt(this)).a)};qNd(7477,1,EAy,nMt);_.ln=function oMt(){jMt(this.a)};var TRc=hAe(7477);xky(hm)(16);\n//# sourceURL=rstudio-16.js\n")
