<header id="title-block-header" class="quarto-title-block default">
<div class="quarto-title">
<h1 class="title">$title$</h1>
$if(subtitle)$
<p class="subtitle lead">$subtitle$</p>
$endif$
$if(categories)$
  $if(quarto-template-params.title-block-categories)$
<div class="quarto-categories">
  $for(categories)$
  <div class="quarto-category">$categories$</div>
  $endfor$
</div>
  $endif$
$endif$
</div>

$if(description)$
<div>
  <div class="description">
    $description$
  </div>
</div>
$endif$

$title-metadata.html()$

</header>