$wnd.rstudio.runAsyncCallback7("function T9s(a){this.a=a}\nfunction Z8s(a,b){this.a=a;this.b=b}\nfunction V8s(a,b){b==null?Q9e(a,T8s.Plc()):Q9e(a,b)}\nfunction k9s(a,b){yi(a.d,new Bkf(false));b9s(a.d,b,0)}\nfunction Wxm(a,b){xzm(a,Epz,'markers_tab_closed',new wL,b)}\nfunction W8s(a,b){var c,d,e,f,g,h,i;d=a.q;Zhe(d.o);for(g=b,h=0,i=g.length;h<i;++h){f=g[h];c=new Z8s(a,f);e=new APd(yPd(new zPd,f).a.a);Tjh(d,new pje(e,c))}}\nfunction b9s(a,b,c){var d;nGj(a.b);V8s(a.c,null);W8s(a.c,mN(gN(b1,1),gly,2,6,[]));if(b.markers){d=b.markers;rGj(a.b,null,d.base_path,d.markers,c,false);W8s(a.c,Dxf(b.names));V8s(a.c,d.name)}}\nfunction j9s(a,b){var c,d,e,f,g;g=b.a.markers_state;if(g.markers){yi(a.d,new Bkf(true));b9s(a.d,b.a.markers_state,b.a.auto_select);e=g.markers;if(e){d=e.markers;if(d.length>0){f=null;c=b.a.auto_select;c==1?(f=d[0]):c==2&&(f=XFj(d));!!f&&T8i(a.b,(zmf(),Rmf(f.path,false,-1,0)),L0e(f.line,f.column),false)}}}else{yi(a.d,new ukf)}}\nfunction P9s(a){try{if(a.c<0)return;if(!a.a.a){a.a.a=aat(a.a.b.a);Liq(a.a,a.a.a)}switch(a.c){case 1:i9s(a.a.a);break;case 2:a.a.a.J.b7b();break;case 3:a.a.a.J.c7b();break;case 4:Wxm(a.a.a.c,new PNh);break;case 5:j9s(a.a.a,a.b[0]);break;case 6:a.a.a.J.d7b();break;case 7:a.a.a.J.e7b();break;case 8:k9s(a.a.a,a.b[0]);}}finally{a.c<=0&&a.b[0]!=null&&a.b[0].ln()}}\nqNd(9087,1,_my,Z8s);_.ln=function $8s(){xE(this.a,this.b)};var FKc=hAe(9087);qNd(439,1,G8y);_.jn=function S9s(){P9s((new T9s(this)).a)};qNd(7234,1,EAy,T9s);_.ln=function U9s(){P9s(this.a)};var SKc=hAe(7234);xky(hm)(7);\n//# sourceURL=rstudio-7.js\n")
